@@ -68,7 +68,7 @@ enum {
   XA_WM_LOCALE_NAME,
   XA_XIM_SERVERS,
 #endif
-#if ENABLE_TRANSPARENCY || ENABLE_PERL
+#if BG_IMAGE_FROM_ROOT || ENABLE_PERL
   XA_XROOTPMAP_ID,
   XA_ESETROOT_PMAP_ID,
 #endif
